const { ipcRenderer } = require('electron');
let tables_data;

function openFile(file_name) {
	$.ajax({
		dataType: "json",
		url: file_name,
		cache: false,
		success: (json, textStatus, jqXHR) => {
			tables_data = json;
			refreshPage();
		}
	});
}

function clearPage() {
	$("#tables").empty();
	$("nav").empty();
	// TODO clear the input values in the details panel
}

function refreshPage() {
	clearPage();
	generateTables();
	showTab(tables_data[0]["id"]+"-tab");
}

function generateTables() {
	for (let table_data of tables_data) {
		const table_id = table_data["id"];
		let a = $("<a></a>")
			.attr("id", table_id+"-nav")
			.addClass("nav-link")
			.attr("href", "#"+table_id)
			.text(table_data["name"])
			.on("click", () => {
				showTab(table_id+"-tab");
			});
		$("nav").append(a);

		let table_div = $("<div></div>")
			.attr("id", table_id+"-tab")
			.addClass("tab-pane table-responsive");

		let table = $("<table></table>")
			.attr("id", table_id)
			.addClass("table table-bordered")
			.append($("<thead></thead>")
				.append($("<tr></tr>")
					.addClass("text-nowrap")))
			.append("<tbody>");

		for (let i=0; i < table_data["col-def"].length; i++) {
			const col_id = table_data["col-def"][i]["field"];
			table.find("thead > tr")
				.append($("<th>")
					.attr("id", table_id+"-"+col_id)
					.text(table_data["col-def"][i]["title"]));
		}

		for (let i=0; i < table_data["rows"].length; i++) {
			const row = table_data["rows"][i];
			const row_id = row["uid"];
			let tr = $("<tr></tr>")
				.attr("id", table_id+"-"+row_id);
			for (let j=0; j < table_data["col-def"].length; j++) {
				const col = table_data["col-def"][j];
				const col_id = col["field"];
				let td = $("<td></td>")
					.attr("id", table_id+"-"+col_id+"-"+row_id)
					.text(row[col_id]);
				tr.append(td);
			}
			table.find("tbody").append(tr);
		}

		table_div.append(table)
		$("#tables").append(table_div);
	}
}

function showTab(tab_name) {
	$("#tables").children().removeClass("show active");
	$("nav").children().removeClass("active");
	$("#"+tab_name).addClass("show active");
	$("#"+tab_name.replace("-tab", "-nav")).addClass("active");
}

function insertColumn(table_id, index) {
	if (index < 0){return;}
	for (let i=0; i < tables_data.length; i++) {
		if (tables_data[i]["id"] === table_id) {
			let col_ids = getColumnIDs(tables_data[i]["col-def"]);
			let col_num = 0;
			for (let id of col_ids) {
				if (col_ids.includes(`column${col_num}`)) {
					col_num++;
				} else {break;}
			}
			let col_id = `column${col_num}`
			tables_data[i]["col-def"].splice(index, 0,{
				"field": col_id,
				"title": "New Column",
				"sortable": false,
				"selectable": false,
			})

			let num_cols = $(`#${table_id} > thead > tr`).children().length;
			if (index >= num_cols) {
				index = num_cols-1;
				$.fn.insert = function (data) {
					this.after(data);
				};
			} else {
				$.fn.insert = function (data) {
					this.before(data);
				};
			}
			$(`#${table_id} > thead > tr > th:nth-child(${index + 1})`)
				.insert($("<th>")
					.attr("id", table_id + "-" + col_id)
					.text("New Column"));

			$(`#${table_id} > tbody > tr`).each((j, tr) => {
				tr = $(tr);
				tr.children(`td:nth-child(${index + 1})`)
					.insert($("<td>")
						.attr("id", table_id+"-"+col_id+"-"+tables_data[i]["rows"][j]["uid"])
						.text(""))
			});
			break;
		}
	}
}

function insertRow(table_id, index) {
	if (index < 0){return;}
	for (let i=0; i < tables_data.length; i++) {
		if (tables_data[i]["id"] === table_id) {
			let row_ids = getRowIDs(tables_data[i]["rows"]);
			let row_num = 0;
			for (let id of row_ids) {
				if (row_ids.includes(`row${row_num}`)) {
					row_num++;
				} else {break;}
			}
			let row_id = `row${row_num}`
			let col_ids = getColumnIDs(tables_data[i]["col-def"]);
			let row_data = {"uid": row_id};
			for (let id of col_ids) {
				row_data[id] = "";
			}
			tables_data[i]["rows"].splice(index, 0, row_data);

			let num_rows = $(`#${table_id} > tbody > tr`).length;
			if (index >= num_rows) {
				index = num_rows-1;
				$.fn.insert = function (data) {
					this.after(data);
				};
			} else {
				$.fn.insert = function (data) {
					this.before(data);
				};
			}
			let tr = $("<tr></tr>")
				.attr("id", table_id+"-"+row_id);
			for (let col_id of col_ids) {
				tr.append($("<td>")
					.attr("id", table_id+"-"+col_id+"-"+row_id)
					.text(""))
			}
			$(`#${table_id} > tbody > tr:nth-child(${index+1})`)
				.insert(tr);
			break;
		}
	}
}

function getColumnIDs(columns) {
	let ids = [];
	for (let col of columns) {
		ids.push(col["field"]);
	}
	return ids;
}

function getRowIDs(rows) {
	let ids = [];
	for (let row of rows) {
		ids.push(row["uid"]);
	}
	return ids;
}

ipcRenderer.on("open-default", (event) => {
	openFile("default.json");
})
